==> mars-src-1.0.1.tar.gz: mars-src-1.0.1/core/src/common/workload_internal_types.h <==
/*
 * Copyright 2008 Sony Corporation of America
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this Library and associated documentation files (the
 * "Library"), to deal in the Library without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Library, and to
 * permit persons to whom the Library is furnished to do so, subject to
 * the following conditions:
 *
 *  The above copyright notice and this permission notice shall be
 *  included in all copies or substantial portions of the Library.
 *
 *  If you modify the Library, you may copy and distribute your modified
 *  version of the Library in object code or as an executable provided
 *  that you also do one of the following:
 *
 *   Accompany the modified version of the Library with the complete
 *   corresponding machine-readable source code for the modified version
 *   of the Library; or,
 *
 *   Accompany the modified version of the Library with a written offer
 *   for a complete machine-readable copy of the corresponding source
 *   code of the modified version of the Library.
 *
 *
 * THE LIBRARY IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * LIBRARY OR THE USE OR OTHER DEALINGS IN THE LIBRARY.
 */

#ifndef MARS_WORKLOAD_INTERNAL_TYPES_H
#define MARS_WORKLOAD_INTERNAL_TYPES_H

#include <stdint.h>

#define MARS_WORKLOAD_ID_NONE			0xffff	/* workload id none */

#define MARS_WORKLOAD_TYPE_NONE			0x00	/* workload undefined */
#define MARS_WORKLOAD_TYPE_TASK			0x01	/* workload type */

#define MARS_WORKLOAD_STATE_NONE		0x00	/* workload undefined */
#define MARS_WORKLOAD_STATE_ADDING		0x01	/* adding now */
#define MARS_WORKLOAD_STATE_REMOVING		0x02	/* removing now */
#define MARS_WORKLOAD_STATE_SCHEDULING		0x03	/* scheduling now */
#define MARS_WORKLOAD_STATE_READY		0x04	/* ready to schedule */
#define MARS_WORKLOAD_STATE_WAITING		0x05	/* waiting for sync */
#define MARS_WORKLOAD_STATE_RUNNING		0x06	/* currently running */
#define MARS_WORKLOAD_STATE_FINISHED		0x07	/* not allow schedule */

#define MARS_WORKLOAD_PRIORITY_MIN		0x00	/* minimum priority */
#define MARS_WORKLOAD_PRIORITY_MAX		0xff	/* maximum priority */

#define MARS_WORKLOAD_COUNTER_MIN		0x0000	/* minimum counter */
#define MARS_WORKLOAD_COUNTER_MAX		0xffff	/* maximum counter */

#define MARS_WORKLOAD_SIGNAL_OFF		0x0	/* signal set off */
#define MARS_WORKLOAD_SIGNAL_ON			0x1	/* signal set on */

#define MARS_WORKLOAD_PER_BLOCK			15	/* wl/block */
#define MARS_WORKLOAD_NUM_BLOCKS		100	/* total blocks */
#define MARS_WORKLOAD_MAX			1500	/* blocks * wl/block */

#define MARS_WORKLOAD_CONTEXT_SIZE		128	/* size of 128 bytes */
#define MARS_WORKLOAD_CONTEXT_ALIGN		128	/* align to 128 bytes */

#define MARS_WORKLOAD_QUEUE_ALIGN		128	/* align to 128 bytes */
#define MARS_WORKLOAD_QUEUE_HEADER_ALIGN	128	/* align to 128 bytes */
#define MARS_WORKLOAD_QUEUE_BLOCK_ALIGN		128	/* align to 128 bytes */

#define MARS_WORKLOAD_QUEUE_FLAG_NONE		0x0	/* no flag set */
#define MARS_WORKLOAD_QUEUE_FLAG_EXIT		0x1	/* exit flag */

/*
 * MARS workload queue block bits
 * ----------------------------------------------------------------------------
 * |[63...60]|[59...56]|[55....48]|[  47  ]|[46....32]|[31.....16]|[15......0]|
 * ----------------------------------------------------------------------------
 * |  4-bits |  4-bits |  8-bits  |  1-bit |  15-bits |  16-bits  |  16-bits  |
 * ----------------------------------------------------------------------------
 * |   TYPE  |  STATE  | PRIORITY | SIGNAL | RESERVED |  WAIT_ID  |  COUNTER  |
 * ----------------------------------------------------------------------------
 */
#define MARS_BITS_SIZE				64

#define MARS_BITS_SHIFT_TYPE			60
#define MARS_BITS_SHIFT_STATE			56
#define MARS_BITS_SHIFT_PRIORITY		48
#define MARS_BITS_SHIFT_SIGNAL			47
#define MARS_BITS_SHIFT_WAIT_ID			16
#define MARS_BITS_SHIFT_COUNTER			0

#define MARS_BITS_MASK_TYPE			0xf000000000000000ULL
#define MARS_BITS_MASK_STATE			0x0f00000000000000ULL
#define MARS_BITS_MASK_PRIORITY			0x00ff000000000000ULL
#define MARS_BITS_MASK_SIGNAL			0x0000800000000000ULL
#define MARS_BITS_MASK_WAIT_ID			0x00000000ffff0000ULL
#define MARS_BITS_MASK_COUNTER			0x000000000000ffffULL

#define MARS_BITS_GET(bits, name)  \
	((*(bits) & MARS_BITS_MASK_##name) >> MARS_BITS_SHIFT_##name)

#define MARS_BITS_SET(bits, name, val) \
	(*bits) = ((*(bits) & ~MARS_BITS_MASK_##name) | \
		((uint64_t)(val) << MARS_BITS_SHIFT_##name))

/* mars workload context */
struct mars_workload_context {
	uint8_t context[MARS_WORKLOAD_CONTEXT_SIZE];
} __attribute__((aligned(MARS_WORKLOAD_CONTEXT_ALIGN)));

/* 128 byte workload queue header structure */
struct mars_workload_queue_header {
	uint64_t queue_ea;
	uint64_t context_ea;
	uint8_t flag;
} __attribute__((aligned(MARS_WORKLOAD_QUEUE_HEADER_ALIGN)));

/* 128 byte workload queue block structure */
struct mars_workload_queue_block {
	uint32_t lock;
	uint32_t pad;
	uint64_t bits[MARS_WORKLOAD_PER_BLOCK];
} __attribute__((aligned(MARS_WORKLOAD_QUEUE_BLOCK_ALIGN)));

/* mars workload queue structure */
struct mars_workload_queue {
	struct mars_workload_queue_header header;
	struct mars_workload_queue_block block[MARS_WORKLOAD_NUM_BLOCKS];
	struct mars_workload_context context[MARS_WORKLOAD_MAX];
} __attribute__((aligned(MARS_WORKLOAD_QUEUE_ALIGN)));

#endif

==> mars-src-1.1.0.tar.gz: mars-src-1.1.0/core/src/common/workload_internal_types.h <==
/*
 * Copyright 2008 Sony Corporation of America
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this Library and associated documentation files (the
 * "Library"), to deal in the Library without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Library, and to
 * permit persons to whom the Library is furnished to do so, subject to
 * the following conditions:
 *
 *  The above copyright notice and this permission notice shall be
 *  included in all copies or substantial portions of the Library.
 *
 *  If you modify the Library, you may copy and distribute your modified
 *  version of the Library in object code or as an executable provided
 *  that you also do one of the following:
 *
 *   Accompany the modified version of the Library with the complete
 *   corresponding machine-readable source code for the modified version
 *   of the Library; or,
 *
 *   Accompany the modified version of the Library with a written offer
 *   for a complete machine-readable copy of the corresponding source
 *   code of the modified version of the Library.
 *
 *
 * THE LIBRARY IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * LIBRARY OR THE USE OR OTHER DEALINGS IN THE LIBRARY.
 */

#ifndef MARS_WORKLOAD_INTERNAL_TYPES_H
#define MARS_WORKLOAD_INTERNAL_TYPES_H

#include <stdint.h>

#define MARS_WORKLOAD_ID_NONE			0xffff	/* workload id none */

#define MARS_WORKLOAD_STATE_NONE		0x00	/* workload undefined */
#define MARS_WORKLOAD_STATE_ADDING		0x01	/* adding now */
#define MARS_WORKLOAD_STATE_REMOVING		0x02	/* removing now */
#define MARS_WORKLOAD_STATE_SCHEDULING		0x03	/* scheduling now */
#define MARS_WORKLOAD_STATE_READY		0x04	/* ready to schedule */
#define MARS_WORKLOAD_STATE_WAITING		0x05	/* waiting for sync */
#define MARS_WORKLOAD_STATE_RUNNING		0x06	/* currently running */
#define MARS_WORKLOAD_STATE_FINISHED		0x07	/* not allow schedule */

#define MARS_WORKLOAD_PRIORITY_MIN		0x00	/* minimum priority */
#define MARS_WORKLOAD_PRIORITY_MAX		0xff	/* maximum priority */

#define MARS_WORKLOAD_COUNTER_MIN		0x0000	/* minimum counter */
#define MARS_WORKLOAD_COUNTER_MAX		0xffff	/* maximum counter */

#define MARS_WORKLOAD_SIGNAL_OFF		0x0	/* signal set off */
#define MARS_WORKLOAD_SIGNAL_ON			0x1	/* signal set on */

#define MARS_WORKLOAD_PER_BLOCK			15	/* wl/block */
#define MARS_WORKLOAD_NUM_BLOCKS		100	/* total blocks */
#define MARS_WORKLOAD_MAX			1500	/* blocks * wl/block */

#define MARS_WORKLOAD_QUEUE_ALIGN		128	/* align to 128 bytes */
#define MARS_WORKLOAD_QUEUE_HEADER_ALIGN	128	/* align to 128 bytes */
#define MARS_WORKLOAD_QUEUE_HEADER_SIZE		128	/* size of 128 bytes */
#define MARS_WORKLOAD_QUEUE_BLOCK_ALIGN		128	/* align to 128 bytes */

#define MARS_WORKLOAD_QUEUE_FLAG_NONE		0x0	/* no flag set */
#define MARS_WORKLOAD_QUEUE_FLAG_EXIT		0x1	/* exit flag */
/*
 * MARS workload queue block bits
 * ------------------------------------------------------------------
 * |[63...60]|[59....52]|[51....33]|[  32  ]|[31.....16]|[15......0]|
 * ------------------------------------------------------------------
 * |  4-bits |  8-bits  |  19-bits |  1-bit |  16-bits  |  16-bits  |
 * ------------------------------------------------------------------
 * |  STATE  | PRIORITY | RESERVED | SIGNAL |  WAIT_ID  |  COUNTER  |
 * ------------------------------------------------------------------
 */
#define MARS_BITS_SIZE				64

#define MARS_BITS_SHIFT_STATE			60
#define MARS_BITS_SHIFT_PRIORITY		52
#define MARS_BITS_SHIFT_SIGNAL			32
#define MARS_BITS_SHIFT_WAIT_ID			16
#define MARS_BITS_SHIFT_COUNTER			0

#define MARS_BITS_MASK_STATE			0xf000000000000000ULL
#define MARS_BITS_MASK_PRIORITY			0x0ff0000000000000ULL
#define MARS_BITS_MASK_SIGNAL			0x0000000100000000ULL
#define MARS_BITS_MASK_WAIT_ID			0x00000000ffff0000ULL
#define MARS_BITS_MASK_COUNTER			0x000000000000ffffULL

#define MARS_BITS_GET(bits, name)  \
	((*(bits) & MARS_BITS_MASK_##name) >> MARS_BITS_SHIFT_##name)

#define MARS_BITS_SET(bits, name, val) \
	(*bits) = ((*(bits) & ~MARS_BITS_MASK_##name) | \
		((uint64_t)(val) << MARS_BITS_SHIFT_##name))

/* 128 byte workload queue header structure */
struct mars_workload_queue_header {
	uint64_t queue_ea;
	uint64_t context_ea;
	uint32_t flag;
	uint8_t pad[MARS_WORKLOAD_QUEUE_HEADER_SIZE - 20];
} __attribute__((aligned(MARS_WORKLOAD_QUEUE_HEADER_ALIGN)));

/* 128 byte workload queue block structure */
struct mars_workload_queue_block {
	uint32_t lock;
	uint32_t pad;
	uint64_t bits[MARS_WORKLOAD_PER_BLOCK];
} __attribute__((aligned(MARS_WORKLOAD_QUEUE_BLOCK_ALIGN)));

/* mars workload queue structure */
struct mars_workload_queue {
	struct mars_workload_queue_header header;
	struct mars_workload_queue_block block[MARS_WORKLOAD_NUM_BLOCKS];
	struct mars_workload_context context[MARS_WORKLOAD_MAX];
} __attribute__((aligned(MARS_WORKLOAD_QUEUE_ALIGN)));

#endif
